Attribute VB_Name = "modResizeKeepRatio"
'********************************************************************
'* Module to resize a hWnd at run-time preserving the aspect ratio: *
'********************************************************************

Option Explicit

Private defWindowProc As Long
Private AspectRatio   As Single

Private Const GWL_WNDPROC As Long = (-4)
Private Const WM_DESTROY = &H2
Private Const WM_SIZING = &H214

'wParam for WM_SIZING message:
Private Const WMSZ_LEFT = 1
Private Const WMSZ_RIGHT = 2
Private Const WMSZ_TOP = 3
Private Const WMSZ_TOPLEFT = 4
Private Const WMSZ_TOPRIGHT = 5
Private Const WMSZ_BOTTOM = 6
Private Const WMSZ_BOTTOMLEFT = 7
Private Const WMSZ_BOTTOMRIGHT = 8

Private Type RECT

  Left   As Long
  Top    As Long
  Right  As Long
  Bottom As Long
  
End Type

Private Declare Function GetWindowLong Lib "user32" Alias "GetWindowLongA" (ByVal hWnd As Long, ByVal nIndex As Long) As Long
Private Declare Function SetWindowLong Lib "user32" Alias "SetWindowLongA" (ByVal hWnd As Long, ByVal nIndex As Long, ByVal dwNewLong As Long) As Long
Private Declare Function CallWindowProc Lib "user32" Alias "CallWindowProcA" (ByVal lpPrevWndFunc As Long, ByVal hWnd As Long, ByVal uMsg As Long, ByVal wParam As Long, ByVal lParam As Long) As Long
Private Declare Sub CopyMemory Lib "kernel32" Alias "RtlMoveMemory" (hpvDest As Any, hpvSource As Any, ByVal cbCopy As Long)

Public Sub Unhook(fhwnd As Long)

  On Error Resume Next
  
  If defWindowProc Then
  
    SetWindowLong fhwnd, GWL_WNDPROC, defWindowProc
    defWindowProc = 0
    
  End If
  
End Sub

Public Sub Hook(fhwnd As Long)

  On Error Resume Next
  defWindowProc = SetWindowLong(fhwnd, GWL_WNDPROC, AddressOf WindowProc)
  
End Sub

Function WindowProc(ByVal hWnd As Long, ByVal uMsg As Long, ByVal wParam As Long, ByVal lParam As Long) As Long

  Dim rc As RECT
  
  On Error Resume Next
  If bResizing = False Then AspectRatio = 0
  
  Select Case uMsg
  
  Case WM_SIZING
  
    'Copy the RECT pointed to in lParam into a RECT structure:
    CopyMemory rc, ByVal lParam, LenB(rc)
    
    'wParam tells which one of the eight possible resizing handles is being used.
    'Set the appropriate RECT member to the size required to maintain aspect ratio, and copy back into the RECT struct for processing by Windows.
    Select Case wParam
    
    Case WMSZ_LEFT
    
      CopyMemory ByVal lParam, rc, LenB(rc)
      WindowProc = 1
      If rc.Bottom <> rc.Top Then AspectRatio = (rc.Right - rc.Left) / (rc.Bottom - rc.Top)
      
    Case WMSZ_RIGHT
    
      CopyMemory ByVal lParam, rc, LenB(rc)
      WindowProc = 1
      If rc.Bottom <> rc.Top Then AspectRatio = (rc.Right - rc.Left) / (rc.Bottom - rc.Top)
      
    Case WMSZ_TOP
    
      CopyMemory ByVal lParam, rc, LenB(rc)
      WindowProc = 1
      If rc.Bottom <> rc.Top Then AspectRatio = (rc.Right - rc.Left) / (rc.Bottom - rc.Top)
      
    Case WMSZ_BOTTOM
    
      CopyMemory ByVal lParam, rc, LenB(rc)
      WindowProc = 1
      If rc.Bottom <> rc.Top Then AspectRatio = (rc.Right - rc.Left) / (rc.Bottom - rc.Top)
      
    Case WMSZ_TOPLEFT
    
      If (AspectRatio = 0) And (rc.Bottom <> rc.Top) Then AspectRatio = (rc.Right - rc.Left) / (rc.Bottom - rc.Top)
      rc.Left = (rc.Top - rc.Bottom) * AspectRatio + (rc.Right)
      CopyMemory ByVal lParam, rc, LenB(rc)
      WindowProc = 1
      If rc.Bottom <> rc.Top Then AspectRatio = (rc.Right - rc.Left) / (rc.Bottom - rc.Top)
      
    Case WMSZ_TOPRIGHT
    
      If (AspectRatio = 0) And (rc.Bottom <> rc.Top) Then AspectRatio = (rc.Right - rc.Left) / (rc.Bottom - rc.Top)
      rc.Right = (rc.Bottom - rc.Top) * AspectRatio + rc.Left
      CopyMemory ByVal lParam, rc, LenB(rc)
      WindowProc = 1
      If rc.Bottom <> rc.Top Then AspectRatio = (rc.Right - rc.Left) / (rc.Bottom - rc.Top)
      
    Case WMSZ_BOTTOMLEFT
    
      If (AspectRatio = 0) And (rc.Bottom <> rc.Top) Then AspectRatio = (rc.Right - rc.Left) / (rc.Bottom - rc.Top)
      If (AspectRatio <> 0) Then rc.Bottom = (rc.Right - rc.Left) / AspectRatio + (rc.Top)
      CopyMemory ByVal lParam, rc, LenB(rc)
      WindowProc = 1
      If rc.Bottom <> rc.Top Then AspectRatio = (rc.Right - rc.Left) / (rc.Bottom - rc.Top)
      
    Case WMSZ_BOTTOMRIGHT
    
      If (AspectRatio = 0) And (rc.Bottom <> rc.Top) Then AspectRatio = (rc.Right - rc.Left) / (rc.Bottom - rc.Top)
      If (AspectRatio <> 0) Then rc.Bottom = (rc.Right - rc.Left) / AspectRatio + rc.Top
      CopyMemory ByVal lParam, rc, LenB(rc)
      WindowProc = 1
      If rc.Bottom <> rc.Top Then AspectRatio = (rc.Right - rc.Left) / (rc.Bottom - rc.Top)
      
    End Select
    
  Case WM_DESTROY:
  
    'Kill subclassing if active:
    If defWindowProc <> 0 Then defWindowProc = 0
    
  End Select
  
  'Process windows messages:
  WindowProc = CallWindowProc(defWindowProc, hWnd, uMsg, wParam, lParam)
  
End Function
